/*******************************************************************
 *
 *		blueTicTacToe
 *		(C) 2003 - Thomas Unterthiner
 *
 *		a classic TicTacToe-Game
 *
 *******************************************************************
 *
 *	CGrid.cpp:	definition for the CGrid-Class
 *
 *	a dynamicaly allocated 2D-Array Template Container (Grid) Class,
 *	can be used for any kind of 2D-Grids
 *
 *******************************************************************
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License as
 *	published by the Free Software Foundation; either version 2 of
 *	the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public
 *	License along with this program; if not, write to the
 *
 *	Free Software Foundation, Inc.,
 *	59 Temple Place,Suite 330,
 *	Boston, MA 02111-1307,
 *	USA
 *
 *******************************************************************
 *
 * for more details about the class, see the CGrid.h
 *
 *******************************************************************/

#ifndef blueTicTacToe_CGrid_cpp
#define blueTicTacToe_CGrid_cpp

#include "CGrid.h"



//	---------------------------------------------------
//	                    PRIVATE MEMBERS
//	---------------------------------------------------

template <class T>
CGrid<T>::CGrid<T>(int Columns, int Rows)
// ctor
{
	this->cols = Columns;
	this->rows = Rows;
	this->size = cols*rows;

	// create a 1dimensional array containing as many elements as a
	// 2D-Matrix with X*Y size.
	p_array = new T[size];
	reset();
};

template <class T>
CGrid<T>::CGrid(const CGrid& rhs)
// copy constructor
{
	this->cols = rhs.columns;
	this->rows = rhs.rows;
	this->size = rhs.size;
	p_array = new T[size];
	reset();
};

template <class T>
CGrid<T>::operator= (const CGrid& rhs)
// assignment-op
{
	this->cols = rhs.columns;
	this->rows = rhs.rows;
	this->size = rhs.size;
	p_array = new T[size];
	reset();
}


template <class T>
CGrid<T>::~CGrid()
// dtor
{
	delete [] p_array;
}






//	---------------------------------------------------
//	                    PUBLIC MEMBERS
//	---------------------------------------------------



template <class T>
T* CGrid<T>::operator[](int neededRow)
// to access array elements:
// array is stored line per line instead of row per row
// RETURN: pointer to the needed "row"... the second []
// usually standing in a CGrid op[] call (after all, this
// is a 2D-Matrix) will then jump to the needed element
// in the array of the remaining elements returned
// (= elements from the needed row till end of all arrays)
{
	return & p_array[neededRow * cols];
};


template <class T>
void CGrid<T>::draw(bool drawGrid, bool drawCoordinateLegend)
// draws the grid with its contents into the console, indented by 1 tab
// TAKES:
//	bool drawGrid: if TRUE, the Grid will be drawn
//	bool drawCoordinateLegend: if TRUE, a Legend will be drawn around the Grid,
{
	
	cout << '\n';
	cout << '\t';

	// draw X-Coordinate-Legend
	if (drawCoordinateLegend)
	{
		cout << ' ' << ' ';	// indent 2 Spaces (they'll be needed for the Y-Coordinate Legend)
		for (int i=0; i<cols; ++i)
		{
			cout << i;
			if (drawGrid) cout << ' ';
		}
		cout << '\n' << '\n';
		cout << '\t';
	}

	for (int i=0; i<size; ++i)
	{
		
		// draw Y-Coordinate Legends every new line
		if (drawCoordinateLegend)
		{
			if (i%cols==0)	// new line
				cout << (i/cols) << ' ';
		}

		if (p_array[i] == 0)
			cout << ' ';
		else
			cout << p_array[i];

		// delimiters
		if (i%cols==cols-1) // line end reached (last row-element is col-1 and not col, because enumeration begins with 0, not with 1)
		{
			cout << '\n' << '\t';
			if (i<size-cols && drawGrid)	// draw row separation (not needed after the last row)
			{
				if (drawCoordinateLegend) cout << ' ' << ' ';	// indent for the 2 Spaces needed by the legend

				// draw a '-' for every col... 
				// cols*(1+drawCoordinateLegend)-1: 
				// iteration goes twice as long, 
				// to make the line long enough
				// for the gridlines as well
				for (short j=0; j < cols*(1+drawGrid)-1; ++j)
				{
					cout << '-';
				}
				cout << '\n';
				cout << '\t';
			}
		}
		else
		{	if (drawGrid) cout << '|';
		}
	}
	cout << '\n';
}


template <class T>
void CGrid<T>::reset()
// sets all grid-fields to 0
{
	if (p_array)
	{
		for (int i=0; i<=size; ++i)
			p_array[i]=0;
	}
}

template <class T>
bool CGrid<T>::doesExist (const T& SearchedObj)
// searchs if a given element exists somewhere on the grid
// RETURNS: TRUE if element exists somehwere, FALSE in any other case
{
	for (int i=0; i<size; ++i)
	{
		if (p_array[i]==SearchedObj) return true;
	}
	return false;
}

#endif

