/*******************************************************************
 *
 *		blueTicTacToe
 *		(C) 2003 - Thomas Unterthiner
 *
 *		a classic TicTacToe-Game
 *
 *******************************************************************
 *
 *	CGrid.h:	declaration for the CGrid-Class
 *
 *	a dynamically allocated  2D-Array Template Container (Grid) Class,
 *	can be used for any kind of 2D-Grids
 *
 *******************************************************************
 *
 *	This program is free software; you can redistribute it and/or 
 *	modify it under the terms of the GNU General Public License as 
 *	published by the Free Software Foundation; either version 2 of 
 *	the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public 
 *	License along with this program; if not, write to the 
 *
 *	Free Software Foundation, Inc.,
 *	59 Temple Place,Suite 330,
 *	Boston, MA 02111-1307,
 *	USA
 *
 *******************************************************************
 *
 * CLASS-DETAILS:
 * --------------
 *
 * 2D grid which is a adynamicly allocated array
 *
 *
 * he elements are stored one after the other in only 1 line
 * instead of line after line, as in normal 2D-Arrays.
 *
 * eg:
 * normal 2D Array size 3x2:
 * 1st Row:	[0]	[1]	[2]
 * 2nd Row:	[0]  	[1]	[2]
 *
 * 1D Array size 3x2:
 * 1st Row:	[0]	[1]	[2]	[3]	[4]	[5]
 * would be:	    1st Row 	     |	    2nd Row
 *
 *
 * Elements can be accessed with an overloaded [] op.
 * an element like "TestArr[2][1]" is accessed this way:
 *
 * 1. jump to the first Element in what would be the needed row,
 *    which is:	row_to_Jump_into x length_of_a_rows (=number of columns)
 *    (in my example, that would be 1x3)
 *
 * 2. return a Pointer to that Element, which can be used
 *    like an array containing all elements starting from the
 *    one pointer points to, till the end of the array
 *
 *    (in my example, that would be Element [0][3] )
 *
 * 3. in that array jump over as much Elements as the 2nd [] says...
 *    (would be 1 array to jump over in my example....)
 *
 *
 * In easier words:
 *	TestArr[2][1]  = 42
 * is the same as:
 *	TestArr.operator[](2) [1] = 42;
 * or even simplier:
 *	int *tmp = TestArr.operator[](0);
 *	tmp[0] = 42;
 *
 *******************************************************************/

#ifndef blueTicTacToe_CGrid_h
#define blueTicTacToe_CGrid_h


#include <iostream>
using namespace std;

template <class T>
class CGrid
{
 private:
	T* p_array;
	int cols;
	int rows;
	int size;

 public:
 	CGrid(int Columns, int Rows);
	// ctor

	CGrid(const CGrid&);
	// copy constructor

	operator= (const CGrid&);
	// assignment-op

	~CGrid();
	// dtor
	
	T* operator[](int neededRow);
	// to access array elements:
	// array is stored line per line instead of row per row
	// RETURN: pointer to the needed "row"... the second []
	// usually standing in a CGrid op[] call (after all, this
	// is a 2D-Matrix) will then jump to the needed element
	// in the array of the remaining elements returned
	// (= elements from the needed row till end of all arrays)

	void draw(bool drawGrid, bool drawCoordinateLegend);
	// draw the grid with its contains into the console
	
	void reset();
	// sets all grid-fields to 0

	bool doesExist (const T& SearchedObj);
	// searchs if a given element exists somewhere on the grid
	// RETURNS: TRUE if element exists somehwere, FALSE in any other case

	// get functions
	int getRows()	{return rows;}
	int getCols()	{return cols;}
	int getSize()	{return size;}

};

#include "CGrid.cpp"	// Template Class' decalarations cannot work without their class-definition

#endif

