/*******************************************************************
 *
 *		blueTicTacToe
 *		(C) 2003 - Thomas Unterthiner
 *
 *		a classic TicTacToe-Game
 *
 *******************************************************************
 *
 *	CTicTacToe.cpp
 *	definition for the CTicTacToe-Class, the main Game-Class
 *
 *******************************************************************
 *
 *	This program is free software; you can redistribute it and/or 
 *	modify it under the terms of the GNU General Public License as 
 *	published by the Free Software Foundation; either version 2 of 
 *	the License, or (at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public 
 *	License along with this program; if not, write to the 
 *
 *	Free Software Foundation, Inc.,
 *	59 Temple Place,Suite 330,
 *	Boston, MA 02111-1307,
 *	USA
 *
 *******************************************************************/

#ifndef blueTicTacToe_CTicTacToe_cpp
#define blueTicTacToe_CTicTacToe_cpp

#include "CTicTacToe.h"


const char Player1Sign='X';
const char Player2Sign='O';


//	-----------------------
//		PRIVATE MEMBERS
//	-----------------------


bool CTicTacToe::makeMove(short Row, short Column, char Sign)
// makes a move (marks a field), if it is still free
// RETURN: false if field could not be marked
{
/*
	cout << Row;
	cout << Column;
	cout << GameGrid[Row][Column];
*/
	// handle illegal moves
	if (Row > grid_size-1		||	//  field-address to high
	    Column > grid_size-1	||
		Row < 0  	||	// field-adress to low
	    Column < 0	||
	    GameGrid[Row][Column])	// field is already marked
	{
		cout << "\n ILLEGAL MOVE! choose another one \n";
		return false;
	}

	else
	{
		GameGrid[Row][Column] = Sign;
		return true;
	}
}


int CTicTacToe::checkForWins()
// checks if someone has already won, by counting the fieldsums in every row/column/diagonal line
// RETURN: PlayerNr of the winner, 0 if nobody has won yet
{
	
/*
	short diagonalSum1 =	GameGrid[0][0] + GameGrid[1][1] + GameGrid[2][2];
	short diagonalSum2 =	GameGrid[0][2] + GameGrid[1][1] + GameGrid[2][0];
*/

	// make diagonal sums
	short diagonalSum1;
	short diagonalSum2;
	if (grid_size%2==1)	// make diagonal sums only if grid size is odd
	{
		diagonalSum1=0;
		diagonalSum2=0;
		for (short i=0; i<grid_size; ++i)
		{
			diagonalSum1 +=	GameGrid[i][i];
			diagonalSum2 +=	GameGrid[i][grid_size-i];
		}
	}
	else	// set to impossible values
	{
		diagonalSum1=-1;
		diagonalSum2=-1;
	}
		
	short HorizontalSum;
	short VerticalSum;

	int Player1WinSum = grid_size*Player1Sign;
	int Player2WinSum = grid_size*Player2Sign;
	
    // storing the var's this way I later have to write the switch-case statement only once
	// going thru this array within a for-iteration
	short* variable[4] = {&diagonalSum1, &diagonalSum2, &HorizontalSum, &VerticalSum};

	for (short row=0; row<grid_size; ++row)
	{
		HorizontalSum=0;
		VerticalSum=0;
		for (short col=0; col<grid_size; ++col)
		{
			HorizontalSum	+= GameGrid[row][col];
			VerticalSum		+= GameGrid[col][row];
		}
     	
		// check the sums (I know I wouldn't have to check the diagonal sums every iteration
		// but writing this way makes the code smaller, and it doesn't take _that_ much speed
		// after all)
		for (short j=0; j<4; ++j)
		{

		/*
			switch (*variable[j])
			{
				case Player1WinSum:
					return 1;
				case Player2WinSum:
					return 2;
			}
		*/
		if (*variable[j]==Player1WinSum)
				return 1;
		else if (*variable[j]==Player2WinSum)
				return 2;
		}
 	}
 	
 	return 0;
}


//	-----------------------
//		PUBLIC MEMBERS
//	-----------------------


void CTicTacToe::play()
// main game-execution
{
	cout << "ok, lets get things started... \n";
	cout << '\n';
	cout << '\n';

	GameGrid.reset();

	short Result=0, MovesCount=0;

	char actualPlayerSign;

	for (short Row=0, Column=0, actualPlayer=1; (!Result && MovesCount < grid_size*grid_size); ++MovesCount)
	{
		actualPlayer=MovesCount%2 + 1;

		if (actualPlayer==1)
			actualPlayerSign=Player1Sign;
		else
			actualPlayerSign=Player2Sign;

		cout << "ok Player " << actualPlayer << ", make your move! \n";	

		do
		{
			cout << "row (1 to "<< grid_size << "): ";
			cin >> Row;

			cout << "column (1 to "<< grid_size << "): ";
			cin >> Column;

			// Grid-adress-enumeration starts with 0, not with 1
			--Row;
			--Column;

		}while ( !makeMove( Row, Column, actualPlayerSign ) );

		Result = checkForWins();
		GameGrid.draw(true, false);
	}

	if (Result)
		cout << "CONGRATULATIONS PLAYER " << Result << ", YOU HAVE WON THE GAME AFTER " << MovesCount << " MOVES! \n";
	else
		cout << "every possible move has been made... no one won.... \n";
	
	return;
}

#endif
